library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity webserver_top is
	port (
	
	 -- ETHERNET DM9000A
      ENET_CMD: OUT STD_LOGIC;
      ENET_CS_N: OUT STD_LOGIC;
      ENET_DATA: INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
      ENET_INT: IN STD_LOGIC;
      ENET_RD_N: OUT STD_LOGIC;
      ENET_RST_N: OUT STD_LOGIC;
      ENET_WR_N: OUT STD_LOGIC;
		ENET_CLK : OUT STD_LOGIC;
	
		-- LCD ports
      LCD_ON  		:	out		std_logic;
      LCD_BLON 	:  out 		std_logic;
      LCD_EN  		:  out 		std_logic;
      LCD_RS  		:  out 		std_logic;
      LCD_RW  		:  out 		std_logic;
      LCD_DATA 	:  inout 	std_logic_vector (7 downto 0);
		
		-- LED port
		LEDG : out std_logic_vector(7 downto 0);
		
		

		-- DRAM ports
		DRAM_CLK : out std_logic;
		DRAM_CKE : out std_logic;
		DRAM_ADDR : out	std_logic_vector(11 DOWNTO 0);
		DRAM_BA_1 : buffer std_logic;
		DRAM_BA_0 : buffer std_logic;
     	DRAM_CS_N : out	std_logic; 
		DRAM_CAS_N : out std_logic; 
		DRAM_RAS_N : out std_logic; 
		DRAM_WE_N : out	std_logic;
     	DRAM_DQ : inout std_logic_vector(15 DOWNTO 0);
     	DRAM_UDQM : buffer std_logic; 
		DRAM_LDQM : buffer std_logic;

      -- CLOCK port
      CLOCK_50 : in std_logic;
				
		
		
		-- 7 Segment PIO
		HEX0, HEX1 : OUT STD_LOGIC_VECTOR(6 DOWNTO 0);
		
		-- CFI Flash
		FL_ADDR : out std_logic_vector(21 downto 0);
		FL_WE_N : out std_logic;
		FL_DQ : inout std_logic_vector(7 downto 0);
		FL_CE_N : out std_logic;
		FL_RST_N : out std_logic;
		FL_OE_N : out std_logic;

			
     	-- RESET key
    	KEY : in std_logic_vector(0 downto 0));
		

end webserver_top;

architecture arch_webserver of webserver_top is
component webserver_sytem is 
           port (
            -- 1) global signals:
                    signal altpll_inst_c0_out : OUT STD_LOGIC;
                    signal altpll_inst_c1_out : OUT STD_LOGIC;
                    signal altpll_inst_c2_out : OUT STD_LOGIC;
                    signal clk_0 : IN STD_LOGIC;
                    signal reset_n : IN STD_LOGIC;

                 -- the_altpll_inst
                    signal locked_from_the_altpll_inst : OUT STD_LOGIC;
                    signal phasedone_from_the_altpll_inst : OUT STD_LOGIC;

                 -- the_dm9000a_inst
                    signal ENET_CMD_from_the_dm9000a_inst : OUT STD_LOGIC;
                    signal ENET_CS_N_from_the_dm9000a_inst : OUT STD_LOGIC;
                    signal ENET_DATA_to_and_from_the_dm9000a_inst : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
                    signal ENET_INT_to_the_dm9000a_inst : IN STD_LOGIC;
                    signal ENET_RD_N_from_the_dm9000a_inst : OUT STD_LOGIC;
                    signal ENET_RST_N_from_the_dm9000a_inst : OUT STD_LOGIC;
                    signal ENET_WR_N_from_the_dm9000a_inst : OUT STD_LOGIC;
						  
						  
                 -- the_lcd_display
                    signal LCD_E_from_the_lcd_display : OUT STD_LOGIC;
                    signal LCD_RS_from_the_lcd_display : OUT STD_LOGIC;
                    signal LCD_RW_from_the_lcd_display : OUT STD_LOGIC;
                    signal LCD_data_to_and_from_the_lcd_display : INOUT STD_LOGIC_VECTOR (7 DOWNTO 0);

                 -- the_led_pio
                    signal out_port_from_the_led_pio : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);

                 -- the_sdram
                    signal zs_addr_from_the_sdram : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
                    signal zs_ba_from_the_sdram : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
                    signal zs_cas_n_from_the_sdram : OUT STD_LOGIC;
                    signal zs_cke_from_the_sdram : OUT STD_LOGIC;
                    signal zs_cs_n_from_the_sdram : OUT STD_LOGIC;
                    signal zs_dq_to_and_from_the_sdram : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
                    signal zs_dqm_from_the_sdram : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
                    signal zs_ras_n_from_the_sdram : OUT STD_LOGIC;
                    signal zs_we_n_from_the_sdram : OUT STD_LOGIC;

						  
						  
                 -- the_seven_seg_pio
                    signal out_port_from_the_seven_seg_pio : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);


                 -- the_tri_state_bridge_avalon_slave
                    signal address_to_the_ext_flash : OUT STD_LOGIC_VECTOR (21 DOWNTO 0);
                    signal data_to_and_from_the_ext_flash : INOUT STD_LOGIC_VECTOR (7 DOWNTO 0);
                    signal read_n_to_the_ext_flash : OUT STD_LOGIC;
                    signal select_n_to_the_ext_flash : OUT STD_LOGIC;
                    signal write_n_to_the_ext_flash : OUT STD_LOGIC
                 );
end component webserver_sytem;

        signal BA : std_logic_vector (1 downto 0);
        signal DQM : std_logic_vector (1 downto 0);
		  signal pll_c1 : std_logic;
        signal pll_locked : std_logic;
        signal pll_phase : std_logic;
		  signal seven_seg_16 : STD_LOGIC_VECTOR(15 downto 0);
		  
begin
 LCD_ON <= '1';

        DRAM_BA_1 <= BA(1);
        DRAM_BA_0 <= BA(0);
        DRAM_UDQM <= DQM(1);
        DRAM_LDQM <= DQM(0);
		  
		  FL_RST_N <= '1';
		  
		HEX0 <= seven_seg_16(6 downto 0);
		HEX1 <= seven_seg_16(14 downto 8);
		  

  --Set us up the Dut
  DUT : webserver_sytem
		port map(
		
		
		  -- the_dm9000a_inst	  
	   ENET_CMD_from_the_dm9000a_inst => ENET_CMD,
		ENET_DATA_to_and_from_the_dm9000a_inst => ENET_DATA,
		ENET_CS_N_from_the_dm9000a_inst => ENET_CS_N,
		ENET_INT_to_the_dm9000a_inst => ENET_INT,
		ENET_RD_N_from_the_dm9000a_inst => ENET_RD_N,
		ENET_RST_N_from_the_dm9000a_inst => ENET_RST_N,
		ENET_WR_N_from_the_dm9000a_inst => ENET_WR_N,
		

		
		-- LCD
      LCD_E_from_the_lcd_display => LCD_EN, 
		LCD_RS_from_the_lcd_display => LCD_RS, 
		LCD_RW_from_the_lcd_display => LCD_RW, 
		LCD_data_to_and_from_the_lcd_display => LCD_DATA,
		
		-- altpll_inst
      altpll_inst_c0_out => DRAM_CLK,
      altpll_inst_c1_out => pll_c1,
		altpll_inst_c2_out => ENET_CLK,
		
		locked_from_the_altpll_inst => pll_locked,
		phasedone_from_the_altpll_inst => pll_phase,
		
	
      
		-- leds
      out_port_from_the_led_pio => LEDG,
		

      
      --sdram
		zs_addr_from_the_sdram => DRAM_ADDR, 
		zs_ba_from_the_sdram => BA, 
		zs_cas_n_from_the_sdram => DRAM_CAS_N, 
		zs_cke_from_the_sdram => DRAM_CKE, 
		zs_cs_n_from_the_sdram => DRAM_CS_N,
		zs_dq_to_and_from_the_sdram => DRAM_DQ, 
		zs_dqm_from_the_sdram => DQM, 
		zs_ras_n_from_the_sdram => DRAM_RAS_N, 
		zs_we_n_from_the_sdram => DRAM_WE_N,
		
		--globals
      clk_0 => CLOCK_50,
      reset_n => KEY(0),
		
		--cfi flash
		address_to_the_ext_flash => FL_ADDR,
      read_n_to_the_ext_flash => FL_OE_N,
      select_n_to_the_ext_flash => FL_CE_N,
      data_to_and_from_the_ext_flash => FL_DQ,
      write_n_to_the_ext_flash => FL_WE_N,
		
		-- seven segment
		out_port_from_the_seven_seg_pio => seven_seg_16
		);
end arch_webserver;
